import { JsonCard } from "@/components/cards/JsonCard";
import SafetyNotesCard from "@/components/cards/SafetyNotesCard";
import { UserCard } from "@/components/cards/UserCard";
import { NavigationToolbar } from "@/components/common/NavigationToolbar";
import { RecentMessages } from "@/components/pages/inspector/RecentMessages";
import { RelevantModerationNotices } from "@/components/pages/inspector/RelevantModerationNotices";
import { RelevantObjects } from "@/components/pages/inspector/RelevantObjects";
import { RelevantReports } from "@/components/pages/inspector/RelevantReports";
import { UserActions } from "@/components/pages/inspector/UserActions";
import { Card, CardHeader } from "@/components/ui/card";
import { Separator } from "@/components/ui/separator";
import {
  fetchBotById,
  fetchBotsByUser,
  fetchMembershipsByUser,
  fetchNoticesByUser,
  fetchReports,
  fetchReportsAgainstUser,
  fetchReportsByUser,
  fetchServers,
  fetchSnapshots,
  fetchStrikesByUser,
  fetchUserById,
  fetchUsersById,
  findDM,
} from "@/lib/db";
import dayjs from "dayjs";
import Link from "next/link";
import { notFound } from "next/navigation";
import { Bot } from "revolt-api";
import relativeTime from "dayjs/plugin/relativeTime";
import { decodeTime } from "ulid";
import { RestrictedUserCard } from "@/components/cards/RestrictedUserCard";

dayjs.extend(relativeTime);

export default async function User({
  params,
}: {
  params: { id: string; type: string };
}) {
  const user = await fetchUserById(params.id);
  if (!user) return notFound();

  const bot = user.bot ? await fetchBotById(user._id) : undefined;
  const botOwner = user.bot ? await fetchUserById(user.bot.owner) : undefined;

  // Fetch strikes and moderation alerts
  const strikes = await fetchStrikesByUser(user._id);
  const notices = await fetchNoticesByUser(user._id);

  // Fetch friends and bots
  const botIds = await fetchBotsByUser(user._id)
    .then((bots) => bots.map((bot) => bot._id))
    .catch(() => []);

  const relevantUsers = await fetchUsersById([
    ...botIds,
    ...(
      user.relations ?? []
    ).map((relation) => relation._id),
  ]);

  // Fetch server memberships
  const serverMemberships = await fetchMembershipsByUser(user._id).catch(
    () => []
  );
  const servers = await fetchServers({
    _id: {
      $in: serverMemberships.map((member) => member._id.server),
    },
  }).catch(() => []);

  // Fetch reports
  const reportsByUser = await fetchReportsByUser(user._id);
  const reportsAgainstUser = await fetchReportsAgainstUser(user._id);

  return (
    <div className="flex flex-col gap-2">
      <NavigationToolbar>Inspecting User</NavigationToolbar>

      <RestrictedUserCard id={user._id} />
      <UserCard user={user} subtitle={`Joined ${dayjs(decodeTime(user._id)).fromNow()} · ${user.status?.text ?? "No status set"}`} />
      <UserActions user={user} bot={bot as Bot} />
      <SafetyNotesCard objectId={user._id} type="user" />

      {user.profile?.content && (
        <Card>
          <CardHeader>
            <p>{user.profile?.content}</p>
          </CardHeader>
        </Card>
      )}

      {botOwner && (
        <Link href={`/panel/inspect/user/${botOwner!._id}`}>
          <UserCard user={botOwner} subtitle="Bot Owner" />
        </Link>
      )}

      <Separator />
      <RelevantModerationNotices
        userId={user._id}
        strikes={strikes}
        notices={notices}
      />

      <Separator />
      <RelevantObjects
        users={relevantUsers}
        servers={servers}
        userId={user._id}
      />

      <Separator />
      <RelevantReports byUser={reportsByUser} forUser={reportsAgainstUser} />

      <Separator />
      <JsonCard obj={user} />
    </div>
  );
}
